@@ -776,34 +776,45 @@ module Senna
     end
   end
 
-  class Client
-    def Client::init()
-      Senna::sen_com_init
-    end
-    def Client::fin()
-      Senna::sen_com_fin
+  class Ctx
+    def Ctx::connect(host='localhost', port=DEFAULT_PORT)
+      handle = Senna::sen_ctx_connect(host, port, 0)
+      return new(handle, nil)
     end
 
-    def Client::open(host='localhost', port=DEFAULT_PORT)
-      handle = Senna::sen_com_client_open(host, port)
-      return new(handle)
+    def Ctx::open(path)
+      db = Senna::sen_db_open(path)
+      raise "sen_db_open(#{path}) failed" unless db
+      handle = Senna::sen_ctx_open(db, 1)
+      return new(handle, db)
     end
 
-    def Client.callback(handle)
-      proc { Senna::sen_com_client_close(handle) }
+    def Ctx::create(path, flags, encoding)
+      db = Senna::sen_db_create(path, flags, encoding)
+      raise "sen_db_create(#{path}) failed" unless db
+      handle = Senna::sen_ctx_open(db, 1)
+      return new(handle, db)
     end
 
-    def initialize(handle)
+    def Ctx.callback(handle, db)
+      proc {
+        Senna::sen_ctx_close(handle)
+        Senna::sen_db_close(db) if db
+      }
+    end
+
+    def initialize(handle, db)
       @handle = handle
-      ObjectSpace.define_finalizer(self, Client.callback(@handle))
+      @db = db
+      ObjectSpace.define_finalizer(self, Ctx.callback(@handle, @db))
     end
 
     def send(message)
-      return Senna::sen_com_client_send(@handle, message, message.length, 0)
+      return Senna::sen_ctx_send(@handle, message, message.length, 0)
     end
 
     def recv()
-      return Senna::sen_com_client_recv(@handle)
+      return Senna::sen_ctx_recv(@handle)
     end
 
     def exec(message)
@@ -817,9 +828,6 @@ module Senna
       res
     end
 
-    def fd()
-      return Senna::sen_com_client_fd(@handle)
-    end
   end
 
   def Senna::get_select_optarg(mode, similarity_threshold_max_interval=0, weight_vector=nil, &func)
