@@ -1,4 +1,4 @@
-/* Copyright(C) 2004 Brazil
+/* Copyright(C) 2004-2007 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,8 @@
 #include "lex.h"
 #include "cache.h"
 #include "store.h"
+#include "ql.h"
+#include "com.h"
 
 #define FOREIGN_KEY     1
 #define FOREIGN_LEXICON 2
@@ -635,6 +637,8 @@ sen_init(void)
 #endif /* USE_AIO */
   sen_default_encoding = sen_strtoenc(SENNA_DEFAULT_ENCODING);
   expand_stack();
+  sen_ql_init_const();
+  sen_com_init();
   return sen_lex_init();
 }
 
@@ -642,6 +646,8 @@ sen_rc
 sen_fin(void)
 {
   sen_lex_fin();
+  sen_logger_fin();
+  sen_com_fin();
   return sen_str_fin();
 }
 
@@ -909,19 +915,21 @@ sen_index_close(sen_index *i)
 sen_rc
 sen_index_remove(const char *path)
 {
+  sen_rc rc;
   char buffer[PATH_MAX];
   if (!path || strlen(path) > PATH_MAX - 8) { return sen_invalid_argument; }
   snprintf(buffer, PATH_MAX, "%s.SEN", path);
-  sen_sym_remove(buffer);
+  if ((rc = sen_sym_remove(buffer))) { goto exit; }
   snprintf(buffer, PATH_MAX, "%s.SEN.i", path);
-  sen_io_remove(buffer); // sen_inv_remove
+  if ((rc = sen_io_remove(buffer))) { goto exit; } // sen_inv_remove
   snprintf(buffer, PATH_MAX, "%s.SEN.i.c", path);
-  sen_io_remove(buffer);
+  if ((rc = sen_io_remove(buffer))) { goto exit; }
   snprintf(buffer, PATH_MAX, "%s.SEN.l", path);
-  sen_sym_remove(buffer);
+  if ((rc = sen_sym_remove(buffer))) { goto exit; }
   snprintf(buffer, PATH_MAX, "%s.SEN.v", path);
-  sen_io_remove(buffer); // sen_vgram_remove
-  return sen_success;
+  rc = sen_io_remove(buffer); // sen_vgram_remove
+exit :
+  return rc;
 }
 
 sen_rc
@@ -1591,7 +1599,7 @@ sen_records_next(sen_records *r, void *keybuf, int bufsize, int *score)
     if (!sen_set_element_info(r->records, r->curr_rec,  (void **)&rid, (void **)&ri)) {
       if (score) { *score = ri->score; }
       if (r->record_unit != sen_rec_userdef) {
-        return sen_sym_key(r->keys, *rid, keybuf, bufsize);
+        return r->keys ? sen_sym_key(r->keys, *rid, keybuf, bufsize) : r->record_size;
       } else {
         if (r->record_size <= bufsize) { memcpy(keybuf, rid, r->record_size); }
         return r->record_size;
@@ -1805,7 +1813,7 @@ sen_records_group(sen_records *r, int limit, sen_group_optarg *optarg)
   }
   while ((rh = sen_set_cursor_next(c, (void **)&key, (void **)&ri))) {
     if (funcp) {
-      if (optarg->func(r, rh, (void **)&gkey, optarg->func_arg)) { continue; }
+      if (optarg->func(r, rh, gkey, optarg->func_arg)) { continue; }
       ekey = key;
     } else {
       gkey = key;
@@ -2379,6 +2387,9 @@ sen_index_term_extract(sen_index *i, const char *string,
   return rc;
 }
 
+/* todo : keep track max_section in sen_inv_header */
+#define MAX_SECTION 2
+
 sen_rc
 sen_index_select(sen_index *i, const char *string, unsigned int string_len,
                  sen_records *r, sen_sel_operator op, sen_select_optarg *optarg)
@@ -2426,8 +2437,9 @@ sen_index_select(sen_index *i, const char *string, unsigned int string_len,
   */
   SEN_LOG(sen_log_info, "n=%d (%s)", n, string);
   if (n == 1 && (*tis)->cursors->n_entries == 1 && op == sen_sel_or
-      && !r->records->n_entries
-      && r->record_unit == sen_rec_document && !r->max_n_subrecs) {
+      && !r->records->n_entries && !r->records->garbages
+      && r->record_unit == sen_rec_document && !r->max_n_subrecs
+      && MAX_SECTION == 1) {
     sen_inv_cursor *c = (*tis)->cursors->bins[0];
     sen_set_array_init(r->records, (*tis)->size + 32768);
     do {
@@ -2729,8 +2741,8 @@ sen_records_heap_next(sen_records_heap *h)
   }
 }
 
-sen_records
-*sen_records_heap_head(sen_records_heap *h)
+sen_records *
+sen_records_heap_head(sen_records_heap *h)
 {
   return h->n_entries ? h->bins[0] : NULL;
 }
@@ -2775,4 +2787,3 @@ sen_info(char **version,
   }
   return sen_success;
 }
-
