@@ -4,12 +4,12 @@
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
-  
+
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
-  
+
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -26,7 +26,7 @@
 /* v08 functions */
 
 void sen_inv_seg_expire08(sen_inv *inv);
-sen_inv * sen_inv_create08(const char *path,  sen_sym *lexicon, 
+sen_inv * sen_inv_create08(const char *path,  sen_sym *lexicon,
                            uint32_t initial_n_segments);
 sen_inv * sen_inv_open08(const char *path, sen_sym *lexicon);
 sen_rc sen_inv_update08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h,
@@ -150,6 +150,7 @@ void
 sen_inv_seg_expire(sen_inv *inv)
 {
   uint32_t expire_threshold = inv->header->initial_n_segments * 2;
+  return; // nop
   if (inv->v08p) { sen_inv_seg_expire08(inv); return; }
   if (inv->seg->nmaps > expire_threshold) {
     uint16_t seg;
@@ -166,7 +167,7 @@ sen_inv_seg_expire(sen_inv *inv)
     }
     for (seg = inv->header->amax; seg; seg--) {
       uint16_t pseg = inv->header->ainfo[seg];
-      if (pseg != SEG_NOT_ASSIGNED) {      
+      if (pseg != SEG_NOT_ASSIGNED) {
         sen_io_mapinfo *info = &inv->seg->maps[pseg];
         uint32_t *pnref = &inv->seg->nrefs[pseg];
         if (info->map && !*pnref) {
@@ -249,76 +250,88 @@ struct sen_inv_buffer {
 
 typedef struct sen_inv_buffer buffer;
 
-inline static sen_rc
+inline static uint16_t
 buffer_open(sen_inv *inv, uint32_t pos, buffer_term **bt, buffer **b)
 {
   byte *p = NULL;
   uint16_t lseg = (uint16_t) (pos >> W_OF_SEGMENT);
   uint16_t pseg = inv->header->binfo[lseg];
-  if (pseg == SEG_NOT_ASSIGNED) { return sen_invalid_argument; }
-  SEN_IO_SEG_REF(inv->seg, pseg, p);
-  if (!p) { return sen_memory_exhausted; }
-  if (b) { *b = (buffer *)p; }
-  if (bt) { *bt = (buffer_term *)(p + (pos & BUFFER_MASK_IN_A_SEGMENT)); }
-  return sen_success;
+  if (pseg != SEG_NOT_ASSIGNED) {
+    SEN_IO_SEG_REF(inv->seg, pseg, p);
+    if (!p) { return SEG_NOT_ASSIGNED; }
+    if (b) { *b = (buffer *)p; }
+    if (bt) { *bt = (buffer_term *)(p + (pos & BUFFER_MASK_IN_A_SEGMENT)); }
+  }
+  return pseg;
 }
 
 inline static sen_rc
-buffer_close(sen_inv *inv, uint32_t pos)
+buffer_close(sen_inv *inv, uint16_t pseg)
 {
-  uint16_t pseg = inv->header->binfo[pos >> W_OF_SEGMENT];
-  if (pseg >= SEN_INV_MAX_SEGMENT) { return sen_invalid_argument; }
+  if (pseg >= SEN_INV_MAX_SEGMENT) {
+    SEN_LOG(sen_log_notice, "invalid pseg buffer_close(%d)", pseg);
+    return sen_invalid_argument;
+  }
   SEN_IO_SEG_UNREF(inv->seg, pseg);
   return sen_success;
 }
 
-inline static int
+inline static uint16_t
 buffer_open_if_capable(sen_inv *inv, int32_t seg, int size, buffer **b)
 {
-  int res, nterms;
+  uint16_t pseg;
   uint32_t pos = ((uint32_t) seg) * SEN_INV_SEGMENT_SIZE;
-  if (buffer_open(inv, pos, NULL, b)) { return 0; }
-  nterms = (*b)->header.nterms - (*b)->header.nterms_void;
-  res = ((nterms < 4096 || 
-         (inv->header->total_chunk_size >> ((nterms >> 8) - 6))
-          > (*b)->header.chunk_size) &&
-         ((*b)->header.buffer_free >= size + sizeof(buffer_term)));
-  if (!res) { buffer_close(inv, pos); }
-  return res;
+  if ((pseg = buffer_open(inv, pos, NULL, b)) != SEG_NOT_ASSIGNED) {
+    uint16_t nterms = (*b)->header.nterms - (*b)->header.nterms_void;
+    if (!((nterms < 4096 ||
+           (inv->header->total_chunk_size >> ((nterms >> 8) - 6))
+           > (*b)->header.chunk_size) &&
+          ((*b)->header.buffer_free >= size + sizeof(buffer_term)))) {
+      buffer_close(inv, pseg);
+      return SEG_NOT_ASSIGNED;
+    }
+  }
+  return pseg;
 }
 
-inline static sen_rc
+inline static uint16_t
 buffer_new(sen_inv *inv, int size, uint32_t *pos,
            buffer_term **bt, buffer_rec **br, buffer **bp, int hint)
 {
   buffer *b;
   uint16_t nseg0 = inv->header->initial_n_segments;
-  uint16_t seg, offset, seg0 = hint % nseg0;
+  uint16_t pseg, seg, offset, seg0 = hint % nseg0;
   uint16_t segmax = (uint16_t) (inv->header->total_chunk_size >> 7) + nseg0;
   if (size + sizeof(buffer_header) + sizeof(buffer_term) > SEN_INV_SEGMENT_SIZE) {
-    return sen_invalid_argument;
+    return SEG_NOT_ASSIGNED;
   }
   // load_all_segments(inv); todo: ainfo and binfo should be inside the header
   for (seg = seg0; seg < segmax; seg += nseg0) {
     if (inv->header->binfo[seg] == SEG_NOT_ASSIGNED) { break; }
-    if (buffer_open_if_capable(inv, seg, size, &b)) { goto exit; }    
+    if ((pseg = buffer_open_if_capable(inv, seg, size, &b)) != SEG_NOT_ASSIGNED) {
+      goto exit;
+    }
   }
   if (seg >= segmax) {
     for (seg = (seg0 + 1) % nseg0; seg != seg0; seg = (seg + 1) % nseg0) {
       if (inv->header->binfo[seg] == SEG_NOT_ASSIGNED) { break; }
-      if (buffer_open_if_capable(inv, seg, size, &b)) { goto exit; }    
+      if ((pseg = buffer_open_if_capable(inv, seg, size, &b)) != SEG_NOT_ASSIGNED) {
+        goto exit;
+      }
     }
     if (seg == seg0) {
       for (seg = nseg0; seg < SEN_INV_MAX_SEGMENT; seg++) {
         if (inv->header->binfo[seg] == SEG_NOT_ASSIGNED) { break; }
-        if (buffer_open_if_capable(inv, seg, size, &b)) { goto exit; }    
+        if ((pseg = buffer_open_if_capable(inv, seg, size, &b)) != SEG_NOT_ASSIGNED) {
+          goto exit;
+        }
       }
     }
   }
   SEN_LOG(sen_log_debug, "inv=%p new seg=%d", inv, seg);
-  if (buffer_segment_new(inv, &seg) || 
-      buffer_open(inv, seg * SEN_INV_SEGMENT_SIZE, NULL, &b)) {
-    return sen_memory_exhausted;
+  if (buffer_segment_new(inv, &seg) ||
+      (pseg = buffer_open(inv, seg * SEN_INV_SEGMENT_SIZE, NULL, &b)) == SEG_NOT_ASSIGNED) {
+    return SEG_NOT_ASSIGNED;
   }
   memset(b, 0, SEN_INV_SEGMENT_SIZE);
   b->header.buffer_free = SEN_INV_SEGMENT_SIZE - sizeof(buffer_header);
@@ -347,7 +360,7 @@ exit :
   *bt = &b->terms[offset];
   *br = (buffer_rec *)(((byte *)&b->terms[b->header.nterms]) + b->header.buffer_free);
   *bp = b;
-  return sen_success;
+  return pseg;
 }
 
 typedef struct {
@@ -449,15 +462,15 @@ sen_inv_check(sen_inv *inv)
   buffer *b;
   uint32_t pos, total_nterms = 0, nerrors = 0;
   uint16_t nseg0 = inv->header->initial_n_segments;
-  uint16_t seg, nsegs = 0;
+  uint16_t pseg, seg, nsegs = 0;
   uint16_t segmax = (uint16_t) (inv->header->total_chunk_size >> 7) + nseg0;
   for (seg = 0; seg < segmax; seg++) {
     if (inv->header->binfo[seg] == SEG_NOT_ASSIGNED) { continue; }
     pos = ((uint32_t) seg) * SEN_INV_SEGMENT_SIZE;
-    if (buffer_open(inv, pos, NULL, &b)) { continue; }
+    if ((pseg = buffer_open(inv, pos, NULL, &b)) == SEG_NOT_ASSIGNED) { continue; }
     nsegs++;
     total_nterms += buffer_check(b, &nerrors);
-    buffer_close(inv, pos);
+    buffer_close(inv, pseg);
   }
   sen_log("sen_inv_check done nsegs=%d total_nterms=%d", nsegs, total_nterms);
   return nerrors;
@@ -781,7 +794,7 @@ sis_deletable(sen_id tid, void *arg)
       return 0;
     }
     array_unref(inv, tid);
-  } 
+  }
   return sym_deletable(tid, h);
 }
 
@@ -800,7 +813,7 @@ sym_delete(sen_inv *inv, uint32_t tid, sen_set *h)
           break;
         }
         array_unref(inv, tid);
-      } 
+      }
       if (!sym_deletable(tid, h)) { break; }
     }
     */
@@ -901,53 +914,51 @@ buffer_flush(sen_inv *inv, uint32_t seg, sen_set *h)
   sen_rc rc = sen_success;
   sen_io_win sw, dw;
   uint8_t *tc, *tp, *dc, *sc = NULL;
-  uint16_t ss, ds;
+  uint16_t ss, ds, pseg;
   uint32_t scn, dcn, max_dest_chunk_size;
   ss = inv->header->binfo[seg];
   if (ss == SEG_NOT_ASSIGNED) { return sen_invalid_format; }
-  if (buffer_open(inv, seg * SEN_INV_SEGMENT_SIZE, NULL, &sb)) {
-    return sen_memory_exhausted;
-  }
+  pseg = buffer_open(inv, seg * SEN_INV_SEGMENT_SIZE, NULL, &sb);
+  if (pseg == SEG_NOT_ASSIGNED) { return sen_memory_exhausted; }
   if ((ds = segment_get(inv)) == SEN_INV_MAX_SEGMENT) {
-    buffer_close(inv, seg * SEN_INV_SEGMENT_SIZE);
+    buffer_close(inv, pseg);
     return sen_memory_exhausted;
   }
   SEN_IO_SEG_REF(inv->seg, ds, db);
   if (!db) {
-    buffer_close(inv, seg * SEN_INV_SEGMENT_SIZE);
+    buffer_close(inv, pseg);
     return sen_memory_exhausted;
   }
   memset(db, 0, SEN_INV_SEGMENT_SIZE);
   max_dest_chunk_size = sb->header.chunk_size + SEN_INV_SEGMENT_SIZE;
   if (!(tc = SEN_MALLOC(max_dest_chunk_size * 2))) {
-    buffer_close(inv, seg * SEN_INV_SEGMENT_SIZE);
+    buffer_close(inv, pseg);
     SEN_IO_SEG_UNREF(inv->seg, ds);
     return sen_memory_exhausted;
-  }  
+  }
   tp = tc + max_dest_chunk_size;
   if (chunk_new(inv, &dcn, max_dest_chunk_size)) {
-    buffer_close(inv, seg * SEN_INV_SEGMENT_SIZE);
+    buffer_close(inv, pseg);
     SEN_IO_SEG_UNREF(inv->seg, ds);
     SEN_FREE(tc);
     return sen_memory_exhausted;
   }
   //  sen_log("db=%p ds=%d sb=%p seg=%d", db, ds, sb, seg);
   if ((scn = sb->header.chunk) != CHUNK_NOT_ASSIGNED) {
-    sc = sen_io_win_map(inv->chunk, &sw, scn, 0, sb->header.chunk_size, sen_io_rdwr);
+    sc = sen_io_win_map(inv->chunk, &sw, scn, 0, sb->header.chunk_size, SEN_IO_COPY);
     if (!sc) {
       SEN_LOG(sen_log_alert, "io_win_map(%d, %d) failed!", scn, sb->header.chunk_size);
-      buffer_close(inv, seg * SEN_INV_SEGMENT_SIZE);
+      buffer_close(inv, pseg);
       SEN_IO_SEG_UNREF(inv->seg, ds);
       SEN_FREE(tc);
       chunk_free(inv, dcn, max_dest_chunk_size);
       return sen_memory_exhausted;
     }
   }
-  // dc = sen_io_win_map(inv->chunk, &dw, dcn, 0, max_dest_chunk_size, sen_io_wronly);
-  dc = sen_io_win_map(inv->chunk, &dw, dcn, 0, max_dest_chunk_size, sen_io_rdwr);
-  if (!dc) { 
+  dc = sen_io_win_map(inv->chunk, &dw, dcn, 0, max_dest_chunk_size, SEN_IO_UPDATE);
+  if (!dc) {
     SEN_LOG(sen_log_alert, "io_win_map(%d, %d) failed!!", dcn, max_dest_chunk_size);
-    buffer_close(inv, seg * SEN_INV_SEGMENT_SIZE);
+    buffer_close(inv, pseg);
     SEN_IO_SEG_UNREF(inv->seg, ds);
     SEN_FREE(tc);
     chunk_free(inv, dcn, max_dest_chunk_size);
@@ -972,7 +983,7 @@ buffer_flush(sen_inv *inv, uint32_t seg, sen_set *h)
         nterms_void++;
         continue;
       }
-      if (sc) { 
+      if (sc) {
         uint32_t o;
         scp = sc + bt->pos_in_chunk;
         sce = scp + bt->size_in_chunk;
@@ -1134,7 +1145,7 @@ buffer_flush(sen_inv *inv, uint32_t seg, sen_set *h)
         if (sgap_ && !(lid.flags & 4)) { lid.flags |= 4; *tcp++ = 0x8e; }
         SEN_C_ENC(dgap_, lid.tf, tcp);
         if (lid.flags & 4) { SEN_B_ENC(sgap_, tcp); }
-        if (lid.flags & 2) { SEN_B_ENC(lid.score, tcp); } 
+        if (lid.flags & 2) { SEN_B_ENC(lid.score, tcp); }
       }
 
       if (ndf) {
@@ -1197,7 +1208,7 @@ buffer_flush(sen_inv *inv, uint32_t seg, sen_set *h)
       inv->header->chunks[dcn + ec++] = 0;
     }
   }
-  buffer_close(inv, seg * SEN_INV_SEGMENT_SIZE);
+  buffer_close(inv, pseg);
   SEN_FREE(tc);
   SEN_IO_SEG_UNREF(inv->seg, ds);
   inv->header->binfo[seg] = ds;
@@ -1336,9 +1347,10 @@ sen_inv_info(sen_inv *inv, off_t *seg_size, off_t *chunk_size)
 sen_rc
 sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int hint)
 {
-  sen_rc r = sen_success;
+  sen_rc rc = sen_success;
   buffer *b;
   uint8_t *bs;
+  uint16_t pseg;
   buffer_rec *br = NULL;
   buffer_term *bt;
   uint32_t pos = 0, size, *a;
@@ -1348,35 +1360,39 @@ sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int h
   // sen_log("key=%d tf=%d pos0=%d rid=%d", key, u->tf, u->pos->pos, u->rid);
   if (!u->tf || !u->sid) { return sen_inv_delete(inv, key, u, h); }
   if (!(a = array_get(inv, key))) { return sen_memory_exhausted; }
-  if (!(bs = encode_rec(u, &size, 0))) { r = sen_memory_exhausted; goto exit; }
+  if (!(bs = encode_rec(u, &size, 0))) { rc = sen_memory_exhausted; goto exit; }
   for (;;) {
     if (*a) {
       if (!(*a & 1)) {
         pos = *a;
-        if ((r = buffer_open(inv, pos, &bt, &b))) { goto exit; }
+        if ((pseg = buffer_open(inv, pos, &bt, &b)) == SEG_NOT_ASSIGNED) {
+          rc = sen_memory_exhausted;
+          goto exit;
+        }
         if (b->header.buffer_free < size) {
           int bfb = b->header.buffer_free;
           SEN_LOG(sen_log_debug, "flushing *a=%d seg=%d(%p) free=%d",
                   *a, *a >> W_OF_SEGMENT, b, b->header.buffer_free);
-          buffer_close(inv, pos);
-          if ((r = buffer_flush(inv, pos >> W_OF_SEGMENT, h))) { goto exit; }
+          buffer_close(inv, pseg);
+          if ((rc = buffer_flush(inv, pos >> W_OF_SEGMENT, h))) { goto exit; }
           if (*a != pos) {
             SEN_LOG(sen_log_debug, "sen_inv_update: *a changed %d->%d", *a, pos);
             continue;
           }
-          if ((r = buffer_open(inv, pos, &bt, &b))) {
+          if ((pseg = buffer_open(inv, pos, &bt, &b)) == SEG_NOT_ASSIGNED) {
             SEN_LOG(sen_log_crit, "buffer not found *a=%d", *a);
-            goto exit; 
+            rc = sen_memory_exhausted;
+            goto exit;
           }
           SEN_LOG(sen_log_debug, "flushed  *a=%d seg=%d(%p) free=%d->%d nterms=%d v=%d",
                   *a, *a >> W_OF_SEGMENT, b, bfb, b->header.buffer_free,
                   b->header.nterms, b->header.nterms_void);
           if (b->header.buffer_free < size) {
-            buffer_close(inv, pos);
+            buffer_close(inv, pseg);
             SEN_LOG(sen_log_crit, "buffer(%d) is full (%d < %d) in sen_inv_update",
                     *a, b->header.buffer_free, size);
             /* todo: must be splitted */
-            r = sen_memory_exhausted;
+            rc = sen_memory_exhausted;
             goto exit;
           }
         }
@@ -1398,10 +1414,11 @@ sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int h
           uint8_t *bs2 = encode_rec(&u2, &size2, 0);
           if (!bs2) {
             SEN_LOG(sen_log_alert, "encode_rec on sen_inv_update failed !");
-            r = sen_memory_exhausted; 
+            rc = sen_memory_exhausted;
             goto exit;
           }
-          if ((r = buffer_new(inv, size + size2, &pos, &bt, &br, &b, hint))) {
+          pseg = buffer_new(inv, size + size2, &pos, &bt, &br, &b, hint);
+          if (pseg == SEG_NOT_ASSIGNED) {
             SEN_FREE(bs2);
             goto exit;
           }
@@ -1410,9 +1427,9 @@ sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int h
           bt->pos_in_chunk = 0;
           bt->size_in_buffer = 0;
           bt->pos_in_buffer = 0;
-          if ((r = buffer_put(b, bt, br, bs2, &u2, size2))) {
+          if ((rc = buffer_put(b, bt, br, bs2, &u2, size2))) {
             SEN_FREE(bs2);
-            buffer_close(inv, pos);
+            buffer_close(inv, pseg);
             goto exit;
           }
           br = (buffer_rec *)(((byte *)br) + size2);
@@ -1429,7 +1446,8 @@ sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int h
       *a = (u->rid << 12) + (u->sid << 1) + 1;
       goto exit;
     } else {
-      if ((r = buffer_new(inv, size, &pos, &bt, &br, &b, hint))) { goto exit; }
+      pseg = buffer_new(inv, size, &pos, &bt, &br, &b, hint);
+      if (pseg == SEG_NOT_ASSIGNED) { goto exit; }
       bt->tid = key;
       bt->size_in_chunk = 0;
       bt->pos_in_chunk = 0;
@@ -1437,8 +1455,8 @@ sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int h
       bt->pos_in_buffer = 0;
     }
   }
-  r = buffer_put(b, bt, br, bs, u, size);
-  buffer_close(inv, pos);
+  rc = buffer_put(b, bt, br, bs, u, size);
+  buffer_close(inv, pseg);
   if (!*a || (*a & 1)) {
     *a = pos;
     sen_sym_pocket_set(inv->lexicon, key, 0);
@@ -1446,14 +1464,15 @@ sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int h
 exit :
   array_unref(inv, key);
   if (bs) { SEN_FREE(bs); }
-  return r;
+  return rc;
 }
 
 sen_rc
 sen_inv_delete(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h)
 {
-  sen_rc r = sen_success;
+  sen_rc rc = sen_success;
   buffer *b;
+  uint16_t pseg;
   uint8_t *bs = NULL;
   buffer_rec *br;
   buffer_term *bt;
@@ -1472,44 +1491,50 @@ sen_inv_delete(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h)
         sym_delete(inv, key, h);
       }
       goto exit;
-    } 
-    if (!(bs = encode_rec(u, &size, 1))) { 
-      r = sen_memory_exhausted;
+    }
+    if (!(bs = encode_rec(u, &size, 1))) {
+      rc = sen_memory_exhausted;
+      goto exit;
+    }
+    if ((pseg = buffer_open(inv, *a, &bt, &b)) == SEG_NOT_ASSIGNED) {
+      rc = sen_memory_exhausted;
       goto exit;
     }
-    if ((r = buffer_open(inv, *a, &bt, &b))) { goto exit; }
     //  sen_log("b->header.buffer_free=%d size=%d", b->header.buffer_free, size);
     if (b->header.buffer_free < size) {
       uint32_t _a = *a;
       SEN_LOG(sen_log_debug, "flushing! b=%p free=%d, seg(%d)", b, b->header.buffer_free, *a >> W_OF_SEGMENT);
-      buffer_close(inv, *a);
-      if ((r = buffer_flush(inv, *a >> W_OF_SEGMENT, h))) { goto exit; }
+      buffer_close(inv, pseg);
+      if ((rc = buffer_flush(inv, *a >> W_OF_SEGMENT, h))) { goto exit; }
       if (*a != _a) {
         SEN_LOG(sen_log_debug, "sen_inv_delete: *a changed %d->%d)", *a, _a);
         continue;
       }
-      if ((r = buffer_open(inv, *a, &bt, &b))) { goto exit; }
+      if ((pseg = buffer_open(inv, *a, &bt, &b)) == SEG_NOT_ASSIGNED) {
+        rc = sen_memory_exhausted;
+        goto exit;
+      }
       SEN_LOG(sen_log_debug, "flushed!  b=%p free=%d, seg(%d)", b, b->header.buffer_free, *a >> W_OF_SEGMENT);
       if (b->header.buffer_free < size) {
         /* todo: must be splitted ? */
         SEN_LOG(sen_log_crit, "buffer(%d) is full (%d < %d) in sen_inv_delete",
                 *a, b->header.buffer_free, size);
-        r = sen_memory_exhausted;
-        buffer_close(inv, *a);
+        rc = sen_memory_exhausted;
+        buffer_close(inv, pseg);
         goto exit;
       }
     }
 
     b->header.buffer_free -= size;
     br = (buffer_rec *)(((byte *)&b->terms[b->header.nterms]) + b->header.buffer_free);
-    r = buffer_put(b, bt, br, bs, u, size);
-    buffer_close(inv, *a);
+    rc = buffer_put(b, bt, br, bs, u, size);
+    buffer_close(inv, pseg);
     break;
   }
 exit :
   array_unref(inv, key);
   if (bs) { SEN_FREE(bs); }
-  return r;
+  return rc;
 }
 
 uint32_t
@@ -1536,7 +1561,7 @@ sen_inv_cursor_open(sen_inv *inv, uint32_t key, int with_pos)
   if (!(c = SEN_MALLOC(sizeof(sen_inv_cursor)))) { goto exit; }
   memset(c, 0, sizeof(sen_inv_cursor));
   c->inv = inv;
-  c->with_pos = with_pos;
+  c->with_pos = (uint16_t) with_pos;
   if (pos & 1) {
     c->stat = 0;
     c->pb.rid = BIT31_12(pos);
@@ -1548,8 +1573,7 @@ sen_inv_cursor_open(sen_inv *inv, uint32_t key, int with_pos)
     uint32_t chunk;
     buffer_term *bt;
     c->pb.rid = 0; c->pb.sid = 0; /* for check */
-    c->buffer_pos = pos;
-    if (buffer_open(inv, pos, &bt, &c->buf)) {
+    if ((c->buffer_pseg = buffer_open(inv, pos, &bt, &c->buf)) == SEG_NOT_ASSIGNED) {
       SEN_FREE(c);
       c = NULL;
       goto exit;
@@ -1558,7 +1582,7 @@ sen_inv_cursor_open(sen_inv *inv, uint32_t key, int with_pos)
       c->cp = sen_io_win_map(inv->chunk, &c->iw,
                              chunk, bt->pos_in_chunk, bt->size_in_chunk, sen_io_rdonly);
       if (!c->cp) {
-        buffer_close(inv, pos);
+        buffer_close(inv, c->buffer_pseg);
         SEN_FREE(c);
         c = NULL;
         goto exit;
@@ -1602,8 +1626,7 @@ sen_inv_cursor_openv1(sen_inv *inv, uint32_t key)
   } else {
     buffer_term *bt;
     c->pb.rid = 0; c->pb.sid = 0;
-    c->buffer_pos = pos;
-    if (buffer_open(inv, pos, &bt, &c->buf)) {
+    if ((c->buffer_pseg = buffer_open(inv, pos, &bt, &c->buf)) == SEG_NOT_ASSIGNED) {
       SEN_FREE(c);
       c = NULL;
       goto exit;
@@ -1795,7 +1818,7 @@ sen_inv_cursor_close(sen_inv_cursor *c)
   }
   if (!c) { return sen_invalid_argument; }
   if (c->cp) { sen_io_win_unmap(&c->iw); }
-  if (c->buf) { buffer_close(c->inv, c->buffer_pos); }
+  if (c->buf) { buffer_close(c->inv, c->buffer_pseg); }
   SEN_FREE(c);
   return sen_success;
 }
@@ -1814,12 +1837,13 @@ sen_inv_estimate_size(sen_inv *inv, uint32_t key)
       res = 1;
     } else {
       buffer *buf;
+      uint16_t pseg;
       buffer_term *bt;
-      if (buffer_open(inv, pos, &bt, &buf)) {
+      if ((pseg = buffer_open(inv, pos, &bt, &buf)) == SEG_NOT_ASSIGNED) {
         res = 0;
       } else {
         res = (bt->size_in_chunk >> 1) + bt->size_in_buffer + 2;
-        buffer_close(inv, pos);
+        buffer_close(inv, pseg);
       }
     }
   } else {
@@ -1839,6 +1863,7 @@ sen_inv_entry_info(sen_inv *inv, unsigned key, unsigned *a, unsigned *pocket,
   buffer *b;
   buffer_term *bt;
   uint32_t *ap;
+  uint16_t pseg;
   if (inv->v08p) {
     return sen_inv_entry_info08(inv, key, a, pocket,
                                 chunk, chunk_size, buffer_free,
@@ -1853,7 +1878,7 @@ sen_inv_entry_info(sen_inv *inv, unsigned key, unsigned *a, unsigned *pocket,
   array_unref(inv, key);
   if (!*a) { return 1; }
   if (*a & 1) { return 2; }
-  if (buffer_open(inv, *a, &bt, &b)) { return 3; }
+  if ((pseg = buffer_open(inv, *a, &bt, &b)) == SEG_NOT_ASSIGNED) { return 3; }
   *chunk = b->header.chunk;
   *chunk_size = b->header.chunk_size;
   *buffer_free = b->header.buffer_free;
@@ -1863,6 +1888,6 @@ sen_inv_entry_info(sen_inv *inv, unsigned key, unsigned *a, unsigned *pocket,
   *pos_in_chunk = bt->pos_in_chunk;
   *size_in_buffer = bt->size_in_buffer;
   *pos_in_buffer = bt->pos_in_buffer;
-  buffer_close(inv, *a);
+  buffer_close(inv, pseg);
   return 4;
 }
