@@ -99,8 +99,8 @@ typedef struct {
   struct sen_inv_buffer *buf;
   uint16_t stat;
   uint16_t nextb;
-  uint32_t buffer_pos;
-  int with_pos;
+  uint16_t buffer_pseg;
+  uint16_t with_pos;
   int flags;
 } sen_inv_cursor;
 
