@@ -394,7 +394,8 @@ sen_io_win_map(sen_io *io, sen_io_win *iw, uint32_t segment,
         return NULL;
       }
     } else {
-      if (!(p = sen_io_seg_ref(io, segment))) { return NULL; }
+      SEN_LOG(sen_log_alert, "nseg == 0! in sen_io_win_map(%p, %u, %u, %u)", io, segment, offset, size);
+      // if (!(p = sen_io_seg_ref(io, segment))) { return NULL; }
     }
     iw->addr = p + offset;
     break;
@@ -735,7 +736,7 @@ sen_io_seg_expire(sen_io *io, uint32_t segno)
     if (nref) {
       SEN_ATOMIC_ADD_EX(pnref, -1, nref);
       if (retry >= SEN_IO_MAX_RETRY) {
-        SEN_LOG(sen_log_crit, "deadlock detected! in sen_io_seg_expire(%p, %u)", io, segno);
+        SEN_LOG(sen_log_crit, "deadlock detected! in sen_io_seg_expire(%p, %u, %u)", io, segno, nref);
         return sen_other_error;
       }
       usleep(1);
@@ -745,7 +746,7 @@ sen_io_seg_expire(sen_io *io, uint32_t segno)
       if (nref > 1) {
         SEN_ATOMIC_ADD_EX(pnref, -(SEN_IO_MAX_REF + 1), nref);
         if (retry >= SEN_IO_MAX_RETRY) {
-          SEN_LOG(sen_log_crit, "deadlock detected!! in sen_io_seg_expire(%p, %u)", io, segno);
+          SEN_LOG(sen_log_crit, "deadlock detected!! in sen_io_seg_expire(%p, %u, %u)", io, segno, nref);
           return sen_other_error;
         }
         usleep(1);
@@ -1078,7 +1079,7 @@ sen_mmap(fileinfo *fi, off_t offset, size_t length)
   }
   res = mmap(NULL, length, PROT_READ|PROT_WRITE, MAP_SHARED, fi->fd, offset);
   if (MAP_FAILED == res) {
-    SEN_LOG(sen_log_alert, "mmap(%zd,%d,%d)=%s <%zd>", length, fi->fd, offset, strerror(errno), mmap_size);
+    SEN_LOG(sen_log_alert, "mmap(%zu,%d,%d)=%s <%zu>", length, fi->fd, offset, strerror(errno), mmap_size);
     return NULL;
   }
   mmap_size += length;
@@ -1097,7 +1098,7 @@ sen_munmap(void *start, size_t length)
   int res;
   res = munmap(start, length);
   if (res) {
-    SEN_LOG(sen_log_alert, "munmap(%p,%zd) failed <%zd>", start, length, mmap_size);
+    SEN_LOG(sen_log_alert, "munmap(%p,%zu) failed <%zu>", start, length, mmap_size);
   } else {
     mmap_size -= length;
   }
