@@ -219,6 +219,15 @@ sen_ngram_next(sen_lex *lex)
 static mecab_t *sole_mecab;
 static sen_mutex sole_mecab_lock;
 
+#define SOLE_MECAB_CONFIRM do {\
+  if (!sole_mecab) {\
+    char *arg[] = {"", "-Owakati"};\
+    MUTEX_LOCK(sole_mecab_lock);\
+    if (!sole_mecab) { sole_mecab = mecab_new(2, arg); }\
+    MUTEX_UNLOCK(sole_mecab_lock);\
+  }\
+} while(0)
+
 inline static sen_lex *
 sen_mecab_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
 {
@@ -228,6 +237,11 @@ sen_mecab_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
   if (!(lex = SEN_MALLOC(sizeof(sen_lex)))) { return NULL; }
   lex->sym = sym;
   // sen_log("(%s)", str);
+  SOLE_MECAB_CONFIRM;
+  if (!sole_mecab) {
+    SEN_LOG(sen_log_alert, "mecab_new failed on sen_mecab_open");
+    return NULL;
+  }
   lex->mecab = sole_mecab;
   lex->buf = NULL;
   // if (!(lex->mecab = mecab_new3())) {
@@ -394,9 +408,9 @@ sen_rc
 sen_lex_init(void)
 {
 #ifndef NO_MECAB
-  char *arg[] = {"", "-Owakati"};
+  // char *arg[] = {"", "-Owakati"};
   // return mecab_load_dictionary(2, arg) ? sen_success : sen_other_error;
-  if (!(sole_mecab = mecab_new(2, arg))) { return sen_other_error; }
+  sole_mecab = NULL;
   MUTEX_INIT(sole_mecab_lock);
 #endif /* NO_MECAB */
   return sen_success;
@@ -406,7 +420,10 @@ sen_rc
 sen_lex_fin(void)
 {
 #ifndef NO_MECAB
-  mecab_destroy(sole_mecab);
+  if (sole_mecab) {
+    mecab_destroy(sole_mecab);
+    sole_mecab = NULL;
+  }
   MUTEX_DESTROY(sole_mecab_lock);
 #endif /* NO_MECAB */
   return sen_success;
@@ -480,3 +497,53 @@ sen_lex_close(sen_lex *lex)
   SEN_FREE(lex);
   return sen_success;
 }
+
+sen_rc
+sen_lex_validate(sen_sym *sym)
+{
+  if (!sym) {
+    SEN_LOG(sen_log_warning, "sym is null on sen_lex_validate");
+    return sen_invalid_argument;
+  }
+#ifndef NO_MECAB
+#ifdef USE_MECAB_DICINFO
+  if ((sym->flags & SEN_INDEX_TOKENIZER_MASK) == SEN_INDEX_MORPH_ANALYSE) {
+    sen_encoding enc;
+    const mecab_dictionary_info_t *di;
+
+    SOLE_MECAB_CONFIRM;
+    if (!sole_mecab) {
+      SEN_LOG(sen_log_alert, "mecab_new failed on sen_lex_validate");
+      return sen_other_error;
+    }
+    di = mecab_dictionary_info(sole_mecab);
+    if (!di || !di->charset) {
+      SEN_LOG(sen_log_alert, "mecab_dictionary_info failed on sen_lex_validate");
+      return sen_other_error;
+    }
+    switch (di->charset[0]) {
+      case 'u':
+        enc = sen_enc_utf8;
+        break;
+      case 'e':
+        enc = sen_enc_euc_jp;
+        break;
+      case 'c': /* cp932 */
+      case 's':
+        enc = sen_enc_sjis;
+        break;
+      default:
+        SEN_LOG(sen_log_alert, "unknown encoding %s on sen_lex_validate", di->charset);
+        return sen_other_error;
+    }
+    if (enc != sym->encoding) {
+      SEN_LOG(sen_log_alert,
+              "dictionary encoding %s is differ from sym encoding %s",
+              di->charset, sen_enctostr(sym->encoding));
+      return sen_other_error;
+    }
+  }
+#endif /* USE_MECAB_DICINFO */
+#endif /* NO_MECAB */
+  return sen_success;
+}
