@@ -743,6 +743,75 @@ sen_query_scan(sen_query *q, const char **strs, unsigned int *str_lens, unsigned
   return sen_success;
 }
 
+/* TODO: delete overlapping logic with exec_query */
+sen_rc
+snip_query(sen_query *q, sen_snip *snip, cell *c, sen_sel_operator op,
+           unsigned int n_tags, int c_but,
+           const char **opentags, unsigned int *opentag_lens,
+           const char **closetags, unsigned int *closetag_lens)
+{
+  cell *e, *ope = NIL;
+  sen_sel_operator op0 = sen_sel_or, *opp = &op0, op1 = q->default_op;
+  while (c != NIL) {
+    POP(e, c);
+    switch (e->type) {
+    case sen_ql_op :
+      ope = e;
+      op1 = ope->u.op.op;
+      continue;
+    case sen_ql_bulk :
+      if (ope != NIL) {
+        q->opt.mode = ope->u.op.mode == -1 ? q->default_mode : ope->u.op.mode;
+      } else {
+        q->opt.mode = q->default_mode;
+      }
+      if (!(c_but ^ (*opp == sen_sel_but))) {
+        sen_rc rc;
+        unsigned int i = snip->cond_len % n_tags;
+        if ((rc = sen_snip_add_cond(snip, e->u.b.value, e->u.b.size,
+                                    opentags[i], opentag_lens[i],
+                                    closetags[i], closetag_lens[i]))) {
+          return rc;
+        }
+      }
+      break;
+    case sen_ql_list :
+      if (*opp == sen_sel_but) { c_but ^= 1; }
+      snip_query(q, snip, e, *opp, n_tags, c_but,
+                 opentags, opentag_lens, closetags, closetag_lens);
+      break;
+    default :
+      SEN_LOG(sen_log_notice, "invalid object assigned in query");
+      break;
+    }
+    opp = &op1;
+    ope = NIL;
+    op1 = q->default_op;
+  }
+  return sen_success;
+}
+
+sen_snip *
+sen_query_snip(sen_query *query, int flags,
+               unsigned int width, unsigned int max_results,
+               unsigned int n_tags,
+               const char **opentags, unsigned int *opentag_lens,
+               const char **closetags, unsigned int *closetag_lens,
+               sen_snip_mapping *mapping)
+{
+  sen_snip *res;
+  if (!(res = sen_snip_open(query->encoding, flags, width, max_results,
+                            NULL, 0, NULL, 0, mapping))) {
+    return NULL;
+  }
+  if (snip_query(query, res, query->expr, sen_sel_or, n_tags, 0,
+                 opentags, opentag_lens, closetags, closetag_lens)) {
+    sen_snip_close(res);
+    return NULL;
+  }
+  return res;
+}
+
 #ifdef DEBUG
 
 static void
