@@ -28,7 +28,7 @@
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 #endif
 
-int
+static int
 sen_bm_check_euc(const unsigned char *x, const size_t y)
 {
   const unsigned char *p;
@@ -36,7 +36,7 @@ sen_bm_check_euc(const unsigned char *x, const size_t y)
   return (int) ((x + y - p) & 1);
 }
 
-int
+static int
 sen_bm_check_sjis(const unsigned char *x, const size_t y)
 {
   const unsigned char *p;
@@ -46,7 +46,8 @@ sen_bm_check_sjis(const unsigned char *x, const size_t y)
   return (int) ((x + y - p) & 1);
 }
 
-void
+/*
+static void
 sen_bm_suffixes(const unsigned char *x, size_t m, size_t *suff)
 {
   size_t f, g;
@@ -67,15 +68,18 @@ sen_bm_suffixes(const unsigned char *x, size_t m, size_t *suff)
     }
   }
 }
+*/
 
-void
+static void
 sen_bm_preBmBc(const unsigned char *x, size_t m, size_t *bmBc)
 {
   size_t i;
-  for (i = 0; i < ASIZE; ++i)
+  for (i = 0; i < ASIZE; ++i) {
     bmBc[i] = m;
-  for (i = 0; i < m - 1; ++i)
+  }
+  for (i = 0; i < m - 1; ++i) {
     bmBc[(unsigned int) x[i]] = m - (i + 1);
+  }
 }
 
 #define SEN_BM_COMPARE \
@@ -120,8 +124,8 @@ sen_bm_preBmBc(const unsigned char *x, size_t m, size_t *bmBc)
   } \
 }
 
-inline void
-sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n)
+void
+sen_bm_tunedbm(snip_cond *cond, sen_nstr *object)
 {
   register unsigned char *limit, ck;
   register const unsigned char *p, *cp;
@@ -131,7 +135,7 @@ sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n)
   unsigned char *y;
   size_t shift, found;
 
-  const size_t m = cond->keyword->norm_blen;
+  const size_t n = object->norm_blen, m = cond->keyword->norm_blen;
 
   y = (unsigned char *) object->norm;
   if (m == 1) {
@@ -143,7 +147,7 @@ sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n)
         SEN_BM_COMPARE;
       }
     }
-    cond->stopflag = 1;
+    cond->stopflag = SNIPCOND_STOP;
     return;
   }
 
@@ -187,8 +191,6 @@ sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n)
   }
   /* limit check + search */
   limit = y + n;
-  delta1 = bmBc[p[-1]];
-  p += delta1;
   while(p <= limit) {
     if (!(delta1 = bmBc[p[-1]])) {
       SEN_BM_BM_COMPARE;
@@ -196,11 +198,10 @@ sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n)
     }
     p += delta1;
   }
-  cond->stopflag = 1;
-
+  cond->stopflag = SNIPCOND_STOP;
 }
 
-size_t
+static size_t
 count_mapped_chars(const char *str, const char *end)
 {
   const char *p;
@@ -228,7 +229,7 @@ count_mapped_chars(const char *str, const char *end)
 }
 
 sen_rc
-snip_cond_close(_snip_cond *cond)
+sen_snip_cond_close(snip_cond *cond)
 {
   if (!cond) {
     return sen_invalid_argument;
@@ -239,19 +240,68 @@ snip_cond_close(_snip_cond *cond)
   return sen_success;
 }
 
+sen_rc
+sen_snip_cond_init(snip_cond *sc, const char *keyword, unsigned int keyword_len,
+                sen_encoding enc, int flags)
+{
+  size_t norm_blen;
+  memset(sc, 0, sizeof(snip_cond));
+  if (flags & SEN_SNIP_NORMALIZE) {
+    if (!(sc->keyword = sen_nstr_open(keyword, keyword_len,
+                                      enc, SEN_STR_REMOVEBLANK))) {
+      SEN_LOG(sen_log_alert, "sen_nstr_open on snip_cond_init failed !");
+      return sen_memory_exhausted;
+    }
+  } else {
+    if (!(sc->keyword = sen_fakenstr_open(keyword, keyword_len,
+                                          enc, SEN_STR_REMOVEBLANK))) {
+      SEN_LOG(sen_log_alert, "sen_fakenstr_open on snip_cond_init failed !");
+      return sen_memory_exhausted;
+    }
+  }
+  norm_blen = sc->keyword->norm_blen; /* byte length, not cond->keyword->length */
+  if (!norm_blen) {
+    sen_snip_cond_close(sc);
+    return sen_invalid_argument;
+  }
+  sen_bm_preBmBc((unsigned char *)sc->keyword->norm, norm_blen, sc->bmBc);
+  sc->shift = sc->bmBc[(unsigned char)sc->keyword->norm[norm_blen - 1]];
+  sc->bmBc[(unsigned char)sc->keyword->norm[norm_blen - 1]] = 0;
+  return sen_success;
+}
+
+void
+sen_snip_cond_reinit(snip_cond *cond)
+{
+  cond->found = 0;
+  cond->last_found = 0;
+  cond->start_offset = 0;
+  cond->end_offset = 0;
+
+  cond->count = 0;
+  cond->stopflag = SNIPCOND_NONSTOP;
+}
+
 sen_rc
 sen_snip_add_cond(sen_snip *snip,
                   const char *keyword, unsigned int keyword_len,
                   const char *opentag, unsigned int opentag_len,
                   const char *closetag, unsigned int closetag_len)
 {
-  _snip_cond *cond = snip->cond + snip->cond_len;
-  size_t norm_key_blen;
+  sen_rc rc;
+  snip_cond *cond = snip->cond + snip->cond_len;
 
   if (!keyword || !*keyword || snip->cond_len >= MAX_SNIP_COND_COUNT) {
     return sen_invalid_argument;
   }
-  memset(cond, 0, sizeof(_snip_cond));
+  if ((rc = sen_snip_cond_init(cond, keyword, keyword_len,
+                               snip->encoding, snip->flags))) {
+    return rc;
+  }
+  if (cond->keyword->norm_blen > snip->width) {
+    sen_snip_cond_close(cond);
+    return sen_invalid_argument;
+  }
   if (opentag) {
     cond->opentag = opentag;
     cond->opentag_len = opentag_len;
@@ -266,37 +316,11 @@ sen_snip_add_cond(sen_snip *snip,
     cond->closetag = snip->defaultclosetag;
     cond->closetag_len = snip->defaultclosetag_len;
   }
-
-  if (snip->flags & SEN_SNIP_NORMALIZE) {
-    cond->keyword = sen_nstr_open(keyword, keyword_len,
-                                  snip->encoding, SEN_STR_REMOVEBLANK);
-    if (!cond->keyword) {
-      SEN_LOG(sen_log_alert, "sen_nstr_open on sen_snip_add_cond failed !");
-      return sen_memory_exhausted;
-    }
-  } else {
-    cond->keyword = sen_fakenstr_open(keyword, keyword_len, snip->encoding, SEN_STR_REMOVEBLANK);
-    if (!cond->keyword) {
-      SEN_LOG(sen_log_alert, "sen_fakenstr_open on sen_snip_add_cond failed !");
-      return sen_memory_exhausted;
-    }
-  }
-  norm_key_blen = cond->keyword->norm_blen; /* byte length, not cond->keyword->length */
-  if (!norm_key_blen || norm_key_blen > snip->width) {
-    snip_cond_close(cond);
-    return sen_invalid_argument;
-  }
-
-  sen_bm_preBmBc((unsigned char *) cond->keyword->norm, norm_key_blen, cond->bmBc);
-  cond->shift = cond->bmBc[
-                (unsigned char) cond->keyword->norm[norm_key_blen - 1]];
-  cond->bmBc[(unsigned char) cond->keyword->norm[norm_key_blen - 1]] = 0;
-
   snip->cond_len++;
   return sen_success;
 }
 
-size_t
+static size_t
 sen_snip_find_firstbyte(const char *string, sen_encoding encoding, size_t offset,
                         size_t doffset)
 {
@@ -327,8 +351,12 @@ sen_snip_open(sen_encoding encoding, int flags, unsigned int width,
               sen_snip_mapping *mapping)
 {
   sen_snip *ret = NULL;
-  if (!(ret = SEN_MALLOC(sizeof(sen_snip))) || max_results > MAX_SNIP_RESULT_COUNT
-      || max_results == 0) {
+  if (!(ret = SEN_MALLOC(sizeof(sen_snip)))) {
+    SEN_LOG(sen_log_alert, "sen_snip allocation failed on sen_snip_open");
+    return NULL;
+  }
+  if (max_results > MAX_SNIP_RESULT_COUNT || max_results == 0) {
+    SEN_LOG(sen_log_warning, "max_results is invalid on sen_snip_open");
     return NULL;
   }
   ret->encoding = encoding;
@@ -348,10 +376,10 @@ sen_snip_open(sen_encoding encoding, int flags, unsigned int width,
   return ret;
 }
 
-sen_rc
+static sen_rc
 exec_clean(sen_snip *snip)
 {
-  _snip_cond *cond;
+  snip_cond *cond;
   if (snip->nstr) {
     sen_nstr_close(snip->nstr);
     snip->nstr = NULL;
@@ -359,13 +387,7 @@ exec_clean(sen_snip *snip)
   snip->tag_count = 0;
   snip->snip_count = 0;
   for (cond = snip->cond; (unsigned int) (cond - snip->cond) < snip->cond_len; cond++) {
-    cond->found = 0;
-    cond->last_found = 0;
-    cond->start_offset = 0;
-    cond->end_offset = 0;
-
-    cond->count = 0;
-    cond->stopflag = 0;
+    sen_snip_cond_reinit(cond);
   }
   return sen_success;
 }
@@ -373,14 +395,14 @@ exec_clean(sen_snip *snip)
 sen_rc
 sen_snip_close(sen_snip *snip)
 {
-  _snip_cond *cond, *cond_end;
+  snip_cond *cond, *cond_end;
   if (!snip) {
     return sen_invalid_argument;
   }
   cond = cond_end = snip->cond;
   cond_end += snip->cond_len;
   for (; cond < cond_end; cond++) {
-    snip_cond_close(cond);
+    sen_snip_cond_close(cond);
   }
   exec_clean(snip);
   SEN_FREE(snip);
@@ -391,7 +413,7 @@ sen_rc
 sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
               unsigned int *nresults, unsigned int *max_tagged_len)
 {
-  size_t nstr_blen, i;
+  size_t i;
 
   exec_clean(snip);
 
@@ -413,10 +435,8 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
     SEN_LOG(sen_log_alert, "sen_nstr_open on sen_snip_exec failed !");
     return sen_memory_exhausted;
   }
-  nstr_blen = snip->nstr->norm_blen; /* byte length, not nstr->length */
-
   for (i = 0; i < snip->cond_len; i++) {
-    sen_bm_TunedBM(snip->cond + i, snip->nstr, nstr_blen);
+    sen_bm_tunedbm(snip->cond + i, snip->nstr);
   }
 
   {
@@ -434,12 +454,14 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
       while (1) {
         size_t min_start_offset = (size_t) -1;
         size_t max_end_offset = 0;
-        _snip_cond *cond = NULL;
+        snip_cond *cond = NULL;
 
         /* get condition which have minimum offset and is not stopped */
         for (i = 0; i < snip->cond_len; i++) {
-          if (snip->cond[i].stopflag == 0 && (min_start_offset > snip->cond[i].start_offset ||
-              (min_start_offset == snip->cond[i].start_offset && max_end_offset < snip->cond[i].end_offset))) {
+          if (snip->cond[i].stopflag == SNIPCOND_NONSTOP &&
+              (min_start_offset > snip->cond[i].start_offset ||
+               (min_start_offset == snip->cond[i].start_offset &&
+                max_end_offset < snip->cond[i].end_offset))) {
             min_start_offset = snip->cond[i].start_offset;
             max_end_offset = snip->cond[i].end_offset;
             cond = &snip->cond[i];
@@ -462,7 +484,7 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
               }
             }
             if (exclude_other_cond) {
-              sen_bm_TunedBM(cond, snip->nstr, nstr_blen);
+              sen_bm_tunedbm(cond, snip->nstr);
               continue;
             }
           }
@@ -475,15 +497,15 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
           /* check nesting to make valid HTML */
           /* ToDo: allow <test><te>te</te><st>st</st></test> */
           if (cond->start_offset < last_tag_end) {
-            sen_bm_TunedBM(cond, snip->nstr, nstr_blen);
+            sen_bm_tunedbm(cond, snip->nstr);
             continue;
           }
         }
         if (cond->end_offset > snip_result->start_offset + snip->width) {
           /* If a keyword gets across a snippet, */
           /* it was skipped and never to be tagged. */
-          cond->stopflag = 2;
-          sen_bm_TunedBM(cond, snip->nstr, nstr_blen);
+          cond->stopflag = SNIPCOND_ACROSS;
+          sen_bm_tunedbm(cond, snip->nstr);
         } else {
           found_cond = 1;
           if (cond->count == 0) {
@@ -502,7 +524,7 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
           if (++snip->tag_count >= MAX_SNIP_TAG_COUNT) {
             break;
           }
-          sen_bm_TunedBM(cond, snip->nstr, nstr_blen);
+          sen_bm_tunedbm(cond, snip->nstr);
         }
       }
       if (!found_cond) {
@@ -540,9 +562,9 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
         break;
       }
       for (i = 0; i < snip->cond_len; i++) {
-        if (snip->cond[i].stopflag != 1) {
+        if (snip->cond[i].stopflag != SNIPCOND_STOP) {
           all_stop = 0;
-          snip->cond[i].stopflag = 0;
+          snip->cond[i].stopflag = SNIPCOND_NONSTOP;
         }
       }
       if (all_stop) {
