@@ -50,6 +50,14 @@ struct _sen_ra {
   struct sen_ra_header *header;
 };
 
+struct sen_ra_header {
+  char idstr[16];
+  unsigned element_size;
+  sen_id curr_max;
+  uint32_t nrecords; /* nrecords is not maintained by default */
+  uint32_t reserved[9];
+};
+
 sen_ra *sen_ra_create(const char *path, unsigned int element_size);
 sen_ra *sen_ra_open(const char *path);
 sen_rc sen_ra_info(sen_ra *ra, unsigned int *element_size, sen_id *curr_max);
@@ -85,29 +93,27 @@ sen_rc sen_ja_replace(sen_ja *ja, sen_id id, sen_ja_einfo *ei);
 
 /**** db ****/
 
-typedef enum {
-  sen_db_nil = 0,
-  sen_db_raw_class,         /* unstructured data class (no slots, not enumerable) */
-  sen_db_structured_class,  /* structured data class (has slots, enumerable) */
-  sen_db_obj_slot,          /* slot storing a reference to a structured object */
-  sen_db_ra_slot,           /* slot storing a flat data, equal or less than 8 bytes */
-  sen_db_ja_slot,           /* slot storing a flat data, more than 8 bytes */
-  sen_db_idx_slot,          /* slot storing an inverted index */
-  sen_db_object,
-  sen_db_records,
-  sen_db_bulk,
-  sen_db_list,
-  sen_db_int,
-  sen_db_float,
-  sen_db_native_method,
-  sen_db_method,
-  sen_db_query
-} sen_db_type;
+#define sen_db_raw_class  0x01 /* unstructured data class (no slots, not enumerable) */
+#define sen_db_class      0x02 /* structured data class (has slots, enumerable) */
+#define sen_db_obj_slot   0x03 /* slot storing a reference to a structured object */
+#define sen_db_ra_slot    0x04 /* slot storing a flat data, equal or less than 8 bytes */
+#define sen_db_ja_slot    0x05 /* slot storing a flat data, more than 8 bytes */
+#define sen_db_idx_slot   0x06 /* slot storing an inverted index */
+#define sen_db_vslot      0x07 /* virtual slot  */
+#define sen_db_pslot      0x08 /* pseudo slot  */
+#define sen_db_rel1       0x09 /* relations the pkey consists of single slot */
+#define sen_db_rel2       0x0a /* relations the pkey consists of two or more slots  */
+
+#define SEN_DB_PSLOT_FLAG 0x80000000
+#define SEN_DB_PSLOT_ID         1
+#define SEN_DB_PSLOT_SCORE      2
+#define SEN_DB_PSLOT_NSUBRECS   3
+#define SEN_DB_PSLOT_MASK 0xff
 
-typedef struct _sen_db sen_db;
 typedef struct _sen_db_store_rel_spec sen_db_store_rel_spec;
 typedef struct _sen_db_store_spec sen_db_store_spec;
 typedef struct _sen_db_store sen_db_store;
+typedef struct _sen_db_trigger sen_db_trigger;
 
 typedef enum {
   sen_db_before_update_trigger = 0,
@@ -122,7 +128,7 @@ struct _sen_db_store_rel_spec {
 
 struct _sen_db_store_spec {
   uint32_t type;
-  uint32_t n_relations;
+  uint32_t n_triggers;
   union {
     struct {
       unsigned int size;
@@ -132,139 +138,70 @@ struct _sen_db_store_spec {
     struct {
       sen_id class;
       unsigned int size;
-      unsigned int collection_type;
+      unsigned int collection_type; // not in used
     } s;
   } u;
-  sen_db_store_rel_spec relations[1];
+  sen_db_store_rel_spec triggers[1];
+};
+
+struct _sen_db {
+  sen_sym *keys;
+  sen_ja *values;
+  sen_set *stores;
+};
+
+struct _sen_db_trigger {
+  sen_db_trigger *next;
+  sen_db_rel_type type;
+  sen_db_store *target;
+};
+
+struct _sen_db_store {
+  uint8_t type;
+  sen_db *db;
+  sen_id id;
+  sen_db_trigger *triggers;
+  union {
+    struct {
+      unsigned int element_size;
+    } bc;
+    struct {
+      sen_sym *keys;
+    } c;
+    struct {
+      sen_id class;
+      sen_ra *ra;
+    } o;
+    struct {
+      sen_id class;
+      sen_ra *ra;
+    } f;
+    struct {
+      sen_id class;
+      sen_ja *ja;
+    } v;
+    struct {
+      sen_id class;
+      sen_index *index;
+    } i;
+  } u;
 };
 
 #define SEN_DB_STORE_SPEC_SIZE(n) \
-  ((intptr_t)(&((sen_db_store_spec *)0)->relations[n]))
-
-sen_db *sen_db_create(const char *path, int flags, sen_encoding encoding);
-sen_db *sen_db_open(const char *path);
-sen_rc sen_db_close(sen_db *s);
+  ((intptr_t)(&((sen_db_store_spec *)0)->triggers[n]))
 
 sen_db_store *sen_db_store_create(sen_db *s, const char *name, sen_db_store_spec *spec);
 sen_db_store *sen_db_store_open(sen_db *s, const char *name);
 sen_rc sen_db_store_remove(sen_db *s, const char *name);
-sen_rc sen_db_store_add_relation(sen_db_store *s, sen_db_store_rel_spec *r);
-sen_rc sen_db_store_del_relation(sen_db_store *s, sen_db_store_rel_spec *r);
+sen_rc sen_db_store_add_trigger(sen_db_store *s, sen_db_store_rel_spec *r);
+sen_rc sen_db_store_del_trigger(sen_db_store *s, sen_db_store_rel_spec *r);
 
-/**** sen_db_obj ****/
-
-typedef struct _sen_db_obj sen_db_obj;
-typedef struct _sen_db_ctx sen_db_ctx;
-typedef struct _sen_db_cont sen_db_cont;
-typedef sen_db_obj *
-sen_db_method_func(sen_db_ctx *, sen_db_obj *, sen_db_cont *);
-typedef sen_db_obj *
-sen_db_parser(sen_db_ctx *, const char *, uint32_t);
-
-struct _sen_db_obj {
-  uint8_t type;
-  uint8_t flags;
-  sen_id class;
-  union {
-    struct {
-      sen_id self;
-    } o;
-    struct {
-      sen_records *records;
-    } r;
-    struct {
-      char *value;
-      uint32_t size;
-    } b;
-    struct {
-      sen_db_obj *car;
-      sen_db_obj *cdr;
-    } l;
-    struct {
-      sen_db_obj *func;
-    } f;
-    struct {
-      sen_db_method_func *func;
-    } n;
-    struct {
-      int32_t i;
-    } i;
-    struct {
-      double d;
-    } d;
-    struct {
-      sen_query *query;
-    } q;
-  } u;
-};
-
-#define SEN_DB_CTX_HEAD 1
-#define SEN_DB_CTX_TAIL 2
-
-#define SEN_DB_CONT_BEGIN(c) if (c) { switch((c)->last) { case 0:
-#define SEN_DB_CONT_WAIT(c,d) \
-  (c)->last = __LINE__; (c)->data = (d); return NULL;\
-  case __LINE__: (d) = (c)->data;
-#define SEN_DB_CONT_END(c) (c)->last = 0; }}
-
-struct _sen_db_cont {
-  uint16_t mode;
-  uint16_t last;
-  void *data;
-};
-
-struct _sen_db_ctx {
-  uint32_t seqno;
-  sen_db *db;
-  sen_set *objects;
-  sen_set *bindings;
-  sen_db_parser *parser;
-  sen_db_method_func *doing;
-  sen_db_cont cont;
-  union {
-    void *ptr;
-    int fd;
-    uint32_t u32;
-    uint64_t u64;
-  } data;
-};
-
-sen_db_ctx *sen_db_ctx_open(sen_db *s, sen_db_parser *parser);
-sen_rc sen_db_ctx_close(sen_db_ctx *c);
-
-sen_rc sen_db_at(sen_db_ctx *c, sen_db_obj *obj,
-                    const char *key, const sen_db_obj *args, sen_db_obj **res);
-sen_rc sen_db_send(sen_db_ctx *c, sen_db_obj *obj,
-                    const char *message, const sen_db_obj *args, sen_db_obj **res);
-sen_db_obj *sen_db_ctx_eval(sen_db_ctx *c, sen_db_obj *sexp);
-void sen_db_obj_inspect(sen_db *db, sen_db_obj *obj, sen_rbuf *buf);
-void sen_db_ctx_def_native_method(sen_db_ctx *c, const char *name,
-                                     sen_db_method_func *func);
-const char *_sen_db_obj_key(sen_db *db, sen_db_obj *obj);
-sen_db_obj *sen_db_ctx_feed(sen_db_ctx *c, char *str, uint32_t str_size, int mode);
-sen_db_obj *sen_db_sexp_parser(sen_db_ctx *c, const char *str, uint32_t str_size);
-
-#define SEN_OBJ2VALUE(o,v,s) ((v) = (o)->u.b.value, (s) = (o)->u.b.size)
-#define SEN_VALUE2OBJ(o,v,s) ((o)->u.b.value = (v), (o)->u.b.size = (s))
-
-#define NILP(c) (!(c) || (c)->type == sen_db_nil)
-#define OBJECTP(c) ((c) && (c)->type == sen_db_object)
-#define RECORDSP(c) ((c) && (c)->type == sen_db_records)
-#define BULKP(c) ((c) && (c)->type == sen_db_bulk)
-#define LISTP(c) ((c) && (c)->type == sen_db_list)
-#define INTP(c) ((c) && (c)->type == sen_db_int)
-#define NATIVE_METHODP(c) ((c) && (c)->type == sen_db_native_method)
-
-#define SETNIL(c) ((c)->type = sen_db_nil)
-#define SETINT(c,v) ((c)->type = sen_db_int, (c)->u.i.i = (v))
-#define SETBULK(c,v,s) ((c)->type = sen_db_bulk, (c)->u.b.value = (v), (c)->u.b.size = (s))
-
-#define CAR(c) (LISTP(c) ? (c)->u.l.car : NULL)
-#define CDR(c) (LISTP(c) ? (c)->u.l.cdr : NULL)
-#define CAAR(c) (CAR(CAR(c)))
-#define CADR(c) (CAR(CDR(c)))
-#define CDAR(c) (CDR(CAR(c)))
-#define CDDR(c) (CDR(CDR(c)))
+sen_db_store *sen_db_store_by_id(sen_db *s, sen_id id);
+sen_db_store *sen_db_slot_class_by_id(sen_db *s, sen_id slot);
+sen_db_store *sen_db_class_slot(sen_db *s, sen_id class, const char *name);
+sen_db_store *sen_db_class_add_slot(sen_db *s, sen_id class, const char *name,
+                                    sen_db_store_spec *spec);
+sen_rc sen_db_class_slotpath(sen_db *s, sen_id class, const char *name, char *buf);
 
 /**** vgram ****/
 
