@@ -133,7 +133,8 @@ sen_str_charlen(const char *str, sen_encoding encoding)
     break;
   case sen_enc_sjis :
     if (*p & 0x80) {
-      if (0xa1 <= *p && *p <= 0xdf) {
+      /* we regard 0xa0 as JIS X 0201 KANA. adjusted to other tools. */
+      if (0xa0 <= *p && *p <= 0xdf) {
         /* hankaku-kana */
         return 1;
       } else if (!(*(p + 1))) {
@@ -179,7 +180,8 @@ sen_str_charlen_nonnull(const char *str, const char *end, sen_encoding encoding)
     break;
   case sen_enc_sjis :
     if (*p & 0x80) {
-      if (0xa1 <= *p && *p <= 0xdf) {
+      /* we regard 0xa0 as JIS X 0201 KANA. adjusted to other tools. */
+      if (0xa0 <= *p && *p <= 0xdf) {
         /* hankaku-kana */
         return 1;
       } else if (++p >= (unsigned char *)end) {
@@ -232,20 +234,31 @@ default_logger_func(int level, const char *time,
 }
 */
 
+static FILE *default_logger_fp = NULL;
+
 static void
 default_logger_func(int level, const char *time, const char *title,
                     const char *msg, const char *location, void *func_arg)
 {
-  static FILE *fp = NULL;
   const char slev[] = " EACewnid-";
-  if (!fp) {
+  if (!default_logger_fp) {
     // mutex_lock
-    fp = fopen(SENNA_LOG_PATH, "a");
+    default_logger_fp = fopen(SENNA_LOG_PATH, "a");
     // mutex_unlock
   }
-  if (fp) {
-    fprintf(fp, "%s|%c|%s %s %s\n", time, *(slev + level), title, msg, location);
-    fflush(fp);
+  if (default_logger_fp) {
+    fprintf(default_logger_fp, "%s|%c|%s %s %s\n",
+            time, *(slev + level), title, msg, location);
+    fflush(default_logger_fp);
+  }
+}
+
+void
+sen_logger_fin(void)
+{
+  if (default_logger_fp) {
+    fclose(default_logger_fp);
+    default_logger_fp = NULL;
   }
 }
 
@@ -405,7 +418,7 @@ static unsigned char symbol[] = {
   '$', 0, 0, '%', '#', '&', '*', '@', 0, 0, 0, 0, 0, 0, 0, 0
 };
 
-inline sen_rc
+inline static sen_rc
 normalize_euc(sen_nstr *nstr)
 {
   static uint16_t hankana[] = {
@@ -1298,7 +1311,10 @@ sen_nstr_open(const char *str, size_t str_len, sen_encoding encoding, int flags)
   sen_rc rc;
   sen_nstr *nstr;
   if (!str || !*str) { return NULL; }
-  if (!(nstr = SEN_MALLOC(sizeof(sen_nstr)))) { return NULL; }
+  if (!(nstr = SEN_MALLOC(sizeof(sen_nstr)))) {
+    SEN_LOG(sen_log_alert, "memory allocation on sen_fakenstr_open failed !");
+    return NULL;
+  }
   nstr->orig = str;
   nstr->orig_blen = str_len;
   nstr->norm = NULL;
@@ -1344,6 +1360,7 @@ sen_fakenstr_open(const char *str, size_t str_len, sen_encoding encoding, int fl
   sen_nstr *nstr;
 
   if (!(nstr = SEN_MALLOC(sizeof(sen_nstr)))) {
+    SEN_LOG(sen_log_alert, "memory allocation on sen_fakenstr_open failed !");
     return NULL;
   }
   if (!(nstr->norm = SEN_MALLOC(str_len + 1))) {
@@ -1357,6 +1374,7 @@ sen_fakenstr_open(const char *str, size_t str_len, sen_encoding encoding, int fl
   nstr->norm[str_len] = '\0';
   nstr->norm_blen = str_len;
   nstr->ctypes = NULL;
+  nstr->flags = flags;
 
   if (flags & SEN_STR_WITH_CHECKS) {
     int_least8_t f = 0;
@@ -1512,17 +1530,20 @@ int
 sen_atoi(const char *nptr, const char *end, const char **rest)
 {
   /* FIXME: INT_MIN is not supported */
-  int v = 0, t, n = 0;
-  if (nptr < end && *nptr == '-') {
-    nptr++;
+  const char *p = nptr;
+  int v = 0, t, n = 0, o = 0;
+  if (p < end && *p == '-') {
+    p++;
     n = 1;
+    o = 1;
   }
-  while (nptr < end && *nptr >= '0' && *nptr <= '9') {
-    t = v * 10 + (*nptr++ - '0');
+  while (p < end && *p >= '0' && *p <= '9') {
+    t = v * 10 + (*p++ - '0');
     if (t < v) { return 0; }
     v = t;
+    o = 0;
   }
-  if (rest) { *rest = nptr; }
+  if (rest) { *rest = o ? nptr : p; }
   return n ? -v : v;
 }
 
@@ -1539,6 +1560,51 @@ sen_atoui(const char *nptr, const char *end, const char **rest)
   return v;
 }
 
+unsigned int
+sen_htoui(const char *nptr, const char *end, const char **rest)
+{
+  unsigned int v = 0, t;
+  while (nptr < end) {
+    switch (*nptr) {
+    case '0' :
+    case '1' :
+    case '2' :
+    case '3' :
+    case '4' :
+    case '5' :
+    case '6' :
+    case '7' :
+    case '8' :
+    case '9' :
+      t = v * 16 + (*nptr++ - '0');
+      break;
+    case 'a' :
+    case 'b' :
+    case 'c' :
+    case 'd' :
+    case 'e' :
+    case 'f' :
+      t = v * 16 + (*nptr++ - 'a') + 10;
+      break;
+    case 'A' :
+    case 'B' :
+    case 'C' :
+    case 'D' :
+    case 'E' :
+    case 'F' :
+      t = v * 16 + (*nptr++ - 'A') + 10;
+      break;
+    default :
+      v = 0; goto exit;
+    }
+    if (t < v) { v = 0; goto exit; }
+    v = t;
+  }
+exit :
+  if (rest) { *rest = nptr; }
+  return v;
+}
+
 void
 sen_str_itoh(unsigned int i, char *p, unsigned int len)
 {
@@ -1575,6 +1641,40 @@ sen_str_itoa(int i, char *p, char *end, char **rest)
   return sen_success;
 }
 
+#define I2B(i) \
+ ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"[(i) & 0x3f])
+
+#define B2I(b) \
+ (((b) < '+' || 'z' < (b)) ? 0xff : "\x3e\xff\xff\xff\x3f\x34\x35\x36\x37\x38\x39\x3a\x3b\x3c\x3d\xff\xff\xff\xff\xff\xff\xff\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\xff\xff\xff\xff\xff\xff\x1a\x1b\x1c\x1d\x1e\x1f\x20\x21\x22\x23\x24\x25\x26\x27\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f\x30\x31\x32\x33"[(b) - '+'])
+
+#define MASK 0x34d34d34
+
+char *
+sen_str_itob(sen_id id, char *p)
+{
+  id ^= MASK;
+  *p++ = I2B(id >> 24);
+  *p++ = I2B(id >> 18);
+  *p++ = I2B(id >> 12);
+  *p++ = I2B(id >> 6);
+  *p++ = I2B(id);
+  return p;
+}
+
+sen_id
+sen_str_btoi(char *b)
+{
+  uint8_t i;
+  sen_id id = 0;
+  int len = 5;
+  while (len--) {
+    char c = *b++;
+    if ((i = B2I(c)) == 0xff) { return 0; }
+    id = (id << 6) + i;
+  }
+  return id ^ MASK;
+}
+
 int
 sen_str_tok(char *str, size_t str_len, char delim, char **tokbuf, int buf_size, char **rest)
 {
@@ -1587,6 +1687,7 @@ sen_str_tok(char *str, size_t str_len, char delim, char **tokbuf, int buf_size,
         break;
       }
       if (delim == *str) {
+        *str = '\0';
         *tok++ = str;
         if (tok == tok_end) { break; }
       }
@@ -1685,7 +1786,7 @@ sen_rbuf_resize(sen_rbuf *buf, size_t newsize)
 {
   char *head;
   newsize = (newsize + (UNIT_MASK)) & ~UNIT_MASK;
-  if (!(head = SEN_REALLOC(buf->head, newsize))) { return sen_memory_exhausted; }
+  if (!(head = SEN_REALLOC(buf->head, newsize + 1))) { return sen_memory_exhausted; }
   buf->curr = head + SEN_RBUF_VSIZE(buf);
   buf->head = head;
   buf->tail = head + newsize;
@@ -1711,6 +1812,17 @@ sen_rbuf_write(sen_rbuf *buf, const char *str, size_t len)
   return rc;
 }
 
+sen_rc
+sen_rbuf_space(sen_rbuf *buf, size_t len)
+{
+  sen_rc rc = sen_success;
+  if (SEN_RBUF_REST(buf) < len) {
+    if ((rc = sen_rbuf_resize(buf, SEN_RBUF_VSIZE(buf) + len))) { return rc; }
+  }
+  buf->curr += len;
+  return rc;
+}
+
 sen_rc
 sen_rbuf_itoa(sen_rbuf *buf, int i)
 {
@@ -1721,10 +1833,69 @@ sen_rbuf_itoa(sen_rbuf *buf, int i)
   return rc;
 }
 
+sen_rc
+sen_rbuf_itoh(sen_rbuf *buf, int i)
+{
+  size_t len = 8;
+  sen_rc rc = sen_success;
+  if (SEN_RBUF_REST(buf) < len) {
+    if ((rc = sen_rbuf_resize(buf, SEN_RBUF_VSIZE(buf) + len))) { return rc; }
+  }
+  sen_str_itoh(i, buf->curr, len);
+  buf->curr += len;
+  return rc;
+}
+
+sen_rc
+sen_rbuf_itob(sen_rbuf *buf, sen_id id)
+{
+  size_t len = 5;
+  sen_rc rc = sen_success;
+  if (SEN_RBUF_REST(buf) < len) {
+    if ((rc = sen_rbuf_resize(buf, SEN_RBUF_VSIZE(buf) + len))) { return rc; }
+  }
+  sen_str_itob(id, buf->curr);
+  buf->curr += len;
+  return rc;
+}
+
+void
+sen_rbuf_str_esc(sen_rbuf *buf, const char *s, int len, sen_encoding encoding)
+{
+  const char *e;
+  unsigned int l;
+  if (len < 0) { len = strlen(s); }
+  SEN_RBUF_PUTC(buf, '"');
+  for (e = s + len; s < e; s += l) {
+    if (!(l = sen_str_charlen_nonnull(s, e, encoding))) { break; }
+    if (l == 1) {
+      switch (*s) {
+      case '\n' :
+        sen_rbuf_write(buf, "\\n", 2);
+        break;
+      case '"' :
+        sen_rbuf_write(buf, "\\\"", 2);
+        break;
+      case '\\' :
+        sen_rbuf_write(buf, "\\\\", 2);
+        break;
+      default :
+        SEN_RBUF_PUTC(buf, *s);
+      }
+    } else {
+      sen_rbuf_write(buf, s, l);
+    }
+  }
+  SEN_RBUF_PUTC(buf, '"');
+}
+
 sen_rc
 sen_rbuf_fin(sen_rbuf *buf)
 {
-  if (buf->head) { SEN_REALLOC(buf->head, 0); }
+  if (buf->head) {
+    SEN_REALLOC(buf->head, 0);
+    buf->head = NULL;
+  }
   return sen_success;
 }
 
