@@ -152,7 +152,7 @@ segment_new(sen_sym *sym, int segtype)
 {\
   if (segno < SEN_SYM_MAX_SEGMENT) {\
     SEN_IO_SEG_REF(io, segno, addr);\
-    SEN_IO_SEG_UNREF(io, segno);\
+    if (addr) { SEN_IO_SEG_UNREF(io, segno); };\
   }\
 }
 
@@ -583,8 +583,7 @@ sen_rc
 sen_sym_remove(const char *path)
 {
   if (!path) { return sen_invalid_argument; }
-  sen_io_remove(path);
-  return sen_success;
+  return sen_io_remove(path);
 }
 
 inline static sen_id
@@ -779,56 +778,72 @@ get_tc(sen_sym *sym, sen_set *h, pat_node *rn, unsigned int len)
   }
 }
 
-sen_set *
-sen_sym_prefix_search(sen_sym *sym, const void *key)
+sen_rc
+sen_sym_prefix_search_with_set(sen_sym *sym, const void *key, sen_set *h)
 {
-  int c;
-  size_t len;
-  sen_id r;
-  pat_node *rn;
-  if (sym->v08p) { return sen_sym_prefix_search08(sym, key); }
-  if (!key || sym->key_size) { return NULL; } /* only for string */
-  len = strlen(key) * 8;
-  rn = pat_at(sym, 0);
-  c = -1;
-  r = rn->r;
+  int c = -1;
+  size_t len = strlen(key) * 8;
+  pat_node *rn = pat_at(sym, 0);
+  sen_id r = rn->r;
   while (r) {
-    if (!(rn = pat_at(sym, r))) { return NULL; }
+    if (!(rn = pat_at(sym, r))) { return sen_invalid_format; }
     if (c < rn->check && len > rn->check) {
       c = rn->check;
       r = nth_bit((uint8_t *)key, c) ? rn->r : rn->l;
       continue;
     }
     if (!memcmp(pat_node_get_key(sym, rn), key, len >> 3)) {
-      sen_set *h = sen_set_open(sizeof(sen_id), 0, 0);
-      SEN_ASSERT(h);
-//      if (!h) { return NULL; }
       sen_set_get(h, &r, NULL);
       if (rn->check >= len) { get_tc(sym, h, rn, (unsigned int)len); }
-      return h;
+      return sen_success;
     }
     break;
   }
-  return NULL;
+  return sen_other_error;
+}
+
+sen_set *
+sen_sym_prefix_search(sen_sym *sym, const void *key)
+{
+  sen_set *h;
+  if (sym->v08p) { return sen_sym_prefix_search08(sym, key); }
+  if (!key || sym->key_size) { return NULL; } /* only for string */
+  h = sen_set_open(sizeof(sen_id), 0, 0);
+  if (sen_sym_prefix_search_with_set(sym, key, h)) {
+    sen_set_close(h);
+    h = NULL;
+  }
+  return h;
+}
+
+sen_rc
+sen_sym_suffix_search_with_set(sen_sym *sym, const void *key, sen_set *h)
+{
+  sen_id r;
+  uint32_t *offset;
+  if ((r = sen_sym_at(sym, key))) {
+    sen_set_get(h, &r, (void **) &offset);
+    *offset = 0;
+    if (sym->flags & SEN_SYM_WITH_SIS) { sis_collect(sym, h, r, 1); }
+    return sen_success;
+  }
+  return sen_other_error;
 }
 
 sen_set *
 sen_sym_suffix_search(sen_sym *sym, const void *key)
 {
-  sen_id r;
-  uint32_t *offset;
+  sen_set *h;
   if (sym->v08p) { return sen_sym_suffix_search08(sym, key); }
   if (!key || sym->key_size) { return NULL; }
-  if ((r = sen_sym_at(sym, key))) {
-    sen_set *h = sen_set_open(sizeof(sen_id), sizeof(uint32_t), 0);
-    SEN_ASSERT(h);
+  h = sen_set_open(sizeof(sen_id), sizeof(uint32_t), 0);
+  SEN_ASSERT(h);
 //    if (!h) { return NULL; }
-    sen_set_get(h, &r, (void **) &offset);
-    *offset = 0;
-    if (sym->flags & SEN_SYM_WITH_SIS) { sis_collect(sym, h, r, 1); }
-    return h;
+  if (sen_sym_suffix_search_with_set(sym, key, h)) {
+    sen_set_close(h);
+    h = NULL;
   }
-  return NULL;
+  return h;
 }
 
 sen_id
