@@ -17,6 +17,7 @@
 
 #include "lib/inv.h"
 #include "lib/sym.h"
+#include "lib/store.h"
 
 #include <stdio.h>
 #include <signal.h>
@@ -237,22 +238,22 @@ check_inv(senchk_file *f)
   printf(" check inv file: '%s'\n", f->path);
 
   if (chkflags & CHK_THROUGH_SEGV || !sigsetjmp(sigret, 1)) {
-    sen_id tid = SEN_SYM_NIL, max_rid, c = 0;
+    sen_id tid = SEN_SYM_NIL, c = 0;
     sen_inv *inv;
     sen_inv_cursor *cur;
     unsigned int tdf = 0, tsf = 0, tnposts = 0;
 
-    if(!(keys = sen_sym_open(f->keyspath))) {
+    if (!(keys = sen_sym_open(f->keyspath))) {
       printf("cannot open keys sym file '%s' "
              "for checking inv file '%s'\n", f->keyspath, f->path);
       e = -1; goto exit;
     }
-    if(!(lex = sen_sym_open(f->lexpath))) {
+    if (!(lex = sen_sym_open(f->lexpath))) {
       printf("cannot open lex sym file '%s' "
              "for checking inv file '%s'\n", f->lexpath, f->path);
       e = -2; goto exit;
     }
-    if(!(i = sen_index_open_with_keys_lexicon(f->path, keys, lex))) {
+    if (!(i = sen_index_open_with_keys_lexicon(f->path, keys, lex))) {
       printf("cannot open inv file '%s'\n", f->path);
       e = -3; goto exit;
     }
@@ -264,7 +265,6 @@ check_inv(senchk_file *f)
     }
 
     inv = i->inv;
-    max_rid = sen_sym_curr_id(i->keys);
     if (chkflags & CHK_DUMP_LEXICON) {
       puts("  Inv records infomation:\n"
            "         tid,        df,        sf,    nposts | term");
@@ -274,13 +274,16 @@ check_inv(senchk_file *f)
         fprintf(stderr, "entries: %d\r", c);
       }
       if (!(cur = sen_inv_cursor_open(inv, tid, 1))) {
+        /* TODO: detect array_at fails or not
         printf("cannot open inv cursor '%s'\n", f->path);
         f->cerr++;
+        */
       } else {
         sen_inv_posting pre = {0, 0, 0, 0, 0, 0};
         unsigned int df = 0, sf = 0, nposts = 0;
-        while(!sen_inv_cursor_next(cur) && f->cerr < MAX_NERR_FILE) {
+        while (!sen_inv_cursor_next(cur) && f->cerr < MAX_NERR_FILE) {
           sen_inv_posting *post = cur->post;
+          sen_id max_rid = sen_sym_curr_id(i->keys);
           if (pre.rid > post->rid || post->rid > max_rid) {
             printf("invalid rid: pre-rid:%d rid:%d max_rid:%d\n",
                    pre.rid, post->rid, max_rid);
@@ -302,9 +305,10 @@ check_inv(senchk_file *f)
               f->cerr++;
             }
             df++;
+            sf++;
           }
           pre.pos = 0;
-          while(!sen_inv_cursor_next_pos(cur) && f->cerr < MAX_NERR_FILE) {
+          while (!sen_inv_cursor_next_pos(cur) && f->cerr < MAX_NERR_FILE) {
             post = cur->post;
             if (pre.pos && pre.pos >= post->pos) {
               printf("invalid position: rid:%d sid:%d pre-pos:%d pos:%d\n",
@@ -339,7 +343,7 @@ check_inv(senchk_file *f)
       c++;
     }
     printf(" checked inv file %s: total %d entiries\n"
-           " total: %d records/%d sections/%d posts\n", f->path, c, tdf, tsf, tnposts);
+           " checked: %d records/%d sections/%d posts\n", f->path, c, tdf, tsf, tnposts);
     e = f->cerr;
   } else {
     /* handle segv */
@@ -419,7 +423,7 @@ senchk_filelist_init(unsigned int max_n_files)
 {
   senchk_filelist *fl;
   if ((fl = SEN_MALLOC(sizeof(senchk_filelist)))) {
-    if ((fl->files = SEN_MALLOC(max_n_files * sizeof(senchk_file)))) {
+    if ((fl->files = SEN_CALLOC(max_n_files * sizeof(senchk_file)))) {
       fl->n_files = 0;
       fl->max_n_files = max_n_files;
       return fl;
@@ -430,7 +434,27 @@ senchk_filelist_init(unsigned int max_n_files)
 }
 
 int
-senchk_filelist_add(senchk_filelist *fl, senchk_filetype type, void *instance)
+senchk_filelist_add(senchk_filelist *fl, senchk_filetype type,
+                    const char *path, const char *keyspath, const char *lexpath)
+{
+  if (fl && path) {
+    senchk_file *f = fl->files + fl->n_files;
+    f->type = type;
+    if (type == type_inv && keyspath && lexpath) {
+      f->keyspath = strdup(keyspath);
+      f->lexpath = strdup(lexpath);
+    } else {
+      f->keyspath = f->lexpath = NULL;
+    }
+    f->path = strdup(path);
+    fl->n_files++;
+    return 1;
+  }
+  return 0;
+}
+
+int
+senchk_filelist_addi(senchk_filelist *fl, senchk_filetype type, void *instance)
 {
   if (fl && instance && fl->max_n_files > fl->n_files) {
     senchk_file *f = fl->files + fl->n_files;
@@ -493,13 +517,70 @@ senchk_filelist_get(senchk_filelist *fl, const char *path)
   int ret = 0;
   if (check_file_existence(path, NULL)) {
     sen_db *db;
-    sen_sym *keys;
     if ((db = sen_db_open(path)) && !segvcount) {
-      if ((keys = sen_sym_open(path))) {
-        /* TODO: implement */
-        puts("db is not supported...");
-        ret = 0;
-        sen_sym_close(keys);
+      sen_id cid = 0;
+      if (senchk_filelist_add(fl, type_sym, path, NULL, NULL)) {
+        ret += 1;
+      }
+      while ((cid = sen_sym_next(db->keys, cid)) != SEN_SYM_NIL) {
+        uint32_t spec_len;
+        char buffer[PATH_MAX];
+        const sen_db_store_spec *spec;
+        if (!(spec = sen_ja_ref(db->values, cid, &spec_len))) {
+          /* TODO: implement */
+          return 0;
+        }
+        gen_pathname(db->keys->io->path, buffer, cid);
+        switch(spec->type) {
+        case sen_db_raw_class :
+          /* not physical file */
+          break;
+        case sen_db_class :
+          /* sym */
+          /* TODO: implement */
+          if (senchk_filelist_add(fl, type_sym, buffer, NULL, NULL)) {
+            ret += 1;
+          }
+          break;
+        case sen_db_obj_slot :
+          /* TODO: implement obj check */
+          /* ra */
+          if (senchk_filelist_add(fl, type_ra, buffer, NULL, NULL)) {
+            ret += 1;
+          }
+          break;
+        case sen_db_ra_slot :
+          /* ra */
+          if (senchk_filelist_add(fl, type_ra, buffer, NULL, NULL)) {
+            ret += 1;
+          }
+          break;
+        case sen_db_ja_slot :
+          /* ja */
+          if (senchk_filelist_add(fl, type_ja, buffer, NULL, NULL)) {
+            ret += 1;
+          }
+          break;
+        case sen_db_idx_slot :
+          /* inv keys lexicon */
+          {
+            sen_db_store *l, *k;
+            if (!(k = sen_db_store_by_id(db, spec->u.s.class))) {
+              /* TODO: implement */
+            }
+            if (!(l = sen_db_slot_class_by_id(db, cid))) {
+              /* TODO: implement */
+            }
+            if (senchk_filelist_add(fl, type_inv, buffer,
+                                    k->u.c.keys->io->path,
+                                    l->u.c.keys->io->path)) {
+              ret += 1;
+            }
+          }
+          break;
+        default:
+          break;
+        }
       }
       sen_db_close(db);
     }
@@ -509,9 +590,9 @@ senchk_filelist_get(senchk_filelist *fl, const char *path)
     sen_index *index;
     if ((index = sen_index_open(path)) && !segvcount) {
       /* TODO: check vgram if exists */
-      if (senchk_filelist_add(fl, type_sym, index->keys) &&
-          senchk_filelist_add(fl, type_sym, index->lexicon) &&
-          senchk_filelist_add(fl, type_inv, index)) {
+      if (senchk_filelist_addi(fl, type_sym, index->keys) &&
+          senchk_filelist_addi(fl, type_sym, index->lexicon) &&
+          senchk_filelist_addi(fl, type_inv, index)) {
         ret = 3;
       }
       sen_index_close(index);
@@ -537,16 +618,23 @@ senchk_filelist_check(senchk_filelist *fl)
           printf("* BROKEN SYM FILE: '%s'\n", f->path);
         }
         break;
-      case type_inv:
-        /* do nothing */
-        break;
       case type_ra:
         /* TODO: implement */
+        show_delimiter(stdout);
+        printf(" check ra file: '%s'\n", f->path);
+        puts("* now checking ra is not implemented...");
         break;
       case type_ja:
         /* TODO: implement */
+        show_delimiter(stdout);
+        printf(" check ja file: '%s'\n", f->path);
+        puts("* now checking ja is not implemented...");
+        break;
+      case type_inv:
+        /* do nothing */
         break;
       default:
+        /* TODO: print error */
         break;
     }
   }
